/* 
* Copyright (C) 2007,2010 Wei Lian <lianwei3 at gmail.com>
*
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/
#include "header.h"
#include "mplayerfork.h"
#include "vlcfork.h"
#include "sopfork.h"
#include "config.h"
#include "pageplay.h"
#include "tabwidget.h"
#include "mainwindow.h"
#include "mystatusbar.h"
//#include "sound.h"
#include "utils.h"

MplayerFork::MplayerFork (TabWidget * tw, QWidget * parent, const char *name):
SopFork (tw, parent, name),
tabwidget (static_cast < TabWidget * >(tw)),
pageplay (static_cast < PagePlay * >(parent))
{
  //cope with bug that existing vlc will mute sound of mplayer
  for (int i = 1; i < tabwidget->count (); i++)
    if (tabwidget->tabToolTip (tabwidget->widget (i)) == "vlc")
      if (static_cast < PagePlay * >(tabwidget->widget (i))->vlcfork->isPlaying)
	{
	  static_cast < PagePlay * >(tabwidget->widget (i))->vlcfork->stopPlayer ();
	}



  player = new Q3Process (this);
  connect (player, SIGNAL (processExited ()), this, SLOT (onPlayerExit ()));

  //cope with bug that existing vlc will mute sound of mplayer
  for (int i = 1; i < tabwidget->count (); i++)
    if (tabwidget->tabToolTip (tabwidget->widget (i)) == "vlc")
      if (!static_cast < PagePlay * >(tabwidget->widget (i))->vlcfork->isPlaying && static_cast < PagePlay * >(tabwidget->widget (i))->vlcfork->snstatus)
	{
	  static_cast < PagePlay * >(tabwidget->widget (i))->vlcfork->forkPlayer ();
	}

}


MplayerFork::~MplayerFork ()
{
  killPlayer ();
}


////fork new process
bool MplayerFork::forkPlayer ()
{
  QStringList
    strlst;
  strlst << "mplayer" << "-slave" << "-quiet" << "-zoom" << "-wid" << QString::number (pageplay->frame->winId ()) << QString ("http://127.0.0.1:").append (outport);

  player->clearArguments ();
  for (int i = 0; i < strlst.size (); i++)
    player->addArgument (strlst.at (i));
  return player->start ();
}

#include <dirent.h>
#include <signal.h>

////kill process
void
MplayerFork::killPlayer ()
{
//   if (!player->isRunning ())
//      return;

  char playerUrl[64] = "http://127.0.0.1:";
  strcat (playerUrl, outport);
  killProcess (playerUrl);
}


/////////////slots
void
MplayerFork::onPlayerExit ()
{
  if (snstatus && !player->isRunning ())
    {
      if (forkPlayer () == TRUE)
	{
	}
      else
	fprintf (stderr, "failed to launch player!\n");
    }

}

void
MplayerFork::parseStdout ()
{
  QString str = QString (sop->readStdout ());

  if (snstatus == 0)
    {
      if (str.contains ("hook_sc") || str.contains ("downloadRate="))
	{
	  //create socket
	  int sockfd = connect_to_server ("127.0.0.1", atoi (outport));
	  if (sockfd < 0)
	    {
	      fprintf (stderr, "failed to create socket, return err %d\n", sockfd);
	      return;
	    }
	  ///launch player
	  if (!player->isRunning ())
	    {
	      if (forkPlayer () == TRUE)
		{
		}
	      else
		fprintf (stderr, "failed to launch player!\n");
	    }
	  ///bind QSocketNotifier to socket
	  snstatus = new QSocketNotifier (sockfd, QSocketNotifier::Read, this);
	  connect (snstatus, SIGNAL (activated (int)), SLOT (streamReady (int)));
	  write (sockfd, "state\ns\n", sizeof ("state\ns\n"));
	  write (sockfd, "state\ns\n", sizeof ("state\ns\n"));
	}
      else if (str.contains ("detect MTU"))
	statusbar->showMessage (tr ("Waiting..."));
    }
}

void
MplayerFork::onSopExit ()
{
  if (forkSop (channelurl, channelname, channeltype))
    {
    }
  else
    fprintf (stderr, "failed to launch sp-sc!\n");
}
